/*
 * Hibernate, Relational Persistence for Idiomatic Java
 *
 * License: GNU Lesser General Public License (LGPL), version 2.1 or later
 * See the lgpl.txt file in the root directory or http://www.gnu.org/licenses/lgpl-2.1.html
 */
package org.hibernate.test.bytecode.enhancement.lazy.proxy;

import java.net.URL;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.Basic;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.LazyGroup;
import org.hibernate.annotations.LazyToOne;
import org.hibernate.annotations.LazyToOneOption;
import org.hibernate.annotations.NaturalId;

/**
 * @author Steve Ebersole
 */
@Entity
@Table( name="web_app" )
public class WebApplication extends BaseEntity {
	private String name;
	private String siteUrl;

	private Set<Activity> activities = new HashSet<>();

	@SuppressWarnings("unused")
	public WebApplication() {
	}

	public WebApplication(Integer id, String siteUrl) {
		super( id );
		this.siteUrl = siteUrl;
	}

	@NaturalId
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Basic( fetch = FetchType.LAZY )
	public String getSiteUrl() {
		return siteUrl;
	}

	public void setSiteUrl(String siteUrl) {
		this.siteUrl = siteUrl;
	}

	@OneToMany(mappedBy="webApplication", fetch= FetchType.LAZY)
	@LazyGroup("app_activity_group")
//	@CollectionType(type="baseutil.technology.hibernate.IskvLinkedSetCollectionType")
	public Set<Activity> getActivities() {
		return activities;
	}

	public void setActivities(Set<Activity> activities) {
		this.activities = activities;
	}
}
